
--
-- Copyright (C) 2024  <fastrgv@gmail.com>
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You may read the full text of the GNU General Public License
-- at <http://www.gnu.org/licenses/>.
--

with gl;  use gl;
with ada.finalization;
with unchecked_deallocation;



package pictobj is -- for textured rectangular exterior, eg. partition wall

-- maps texture coordinates UV to cover the full extent of a side;
-- thus the whole *.png file is visible

type pictangle is tagged private;

--legacy version...draws all 6 exterior faces:
procedure osetrect( rect: pictangle;
xc,yc,zc, xr,yr,zr : float;
xm,xp, ym,yp, zm,zp : out float
);

--draws a single face:
procedure setrect( rect: pictangle;
face: integer; -- 1..6 => xm,xp,ym,yp,zm,zp
xc,yc,zc, xr,yr,zr : float;
xm,xp, ym,yp, zm,zp : out float
);

--draws 2 opposing faces:
procedure setrect2( rect: pictangle;
face: integer; -- 1..6 => xm,xp,ym,yp,zm,zp
xc,yc,zc, xr,yr,zr : float;
xm,xp, ym,yp, zm,zp : out float
);


--draws a single face BUT with texture scaling: U only
procedure setrect( rect: pictangle;
face: integer; -- 1..6 => xm,xp,ym,yp,zm,zp
xc,yc,zc, xr,yr,zr : float;
su : float
);

--draws a single face BUT with texture scaling: U & V
procedure setrect( rect: pictangle;
face: integer; -- 1..6 => xm,xp,ym,yp,zm,zp
xc,yc,zc, xr,yr,zr : float;
su, sv : float
);


procedure draw( rect: pictangle; vertbuff, uvbuff, elembuff : gluint );

--use this for lighting effects
procedure ldraw( rect: pictangle; vertbuff, uvbuff,normbuff, elembuff : gluint );

private

nvert : constant integer := 72;
nuv : constant integer := 48;
nelm : constant integer := 36;

type varray is array(1..nvert) of float;
type vap is access varray;

type tarray is array(1..nuv) of float;
type tap is access tarray;

type earray is array(1..nelm) of glushort;
type eap is access earray;

type pictangle is new ada.finalization.controlled with record
	vert, norm : vap;
	txuv : tap;
	elem : eap;
end record;

sides6: boolean := false;


procedure initialize(rect: in out pictangle);
procedure finalize(rect: in out pictangle);

end pictobj;

