
--
-- Copyright (C) 2024  <fastrgv@gmail.com>
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You may read the full text of the GNU General Public License
-- at <http://www.gnu.org/licenses/>.
--

with gl;  use gl;
with ada.finalization;
with unchecked_deallocation;



package rectobj is 
-- for textured rectangular exteriors, eg. partition wall

-- Intended for objects bigger than 1 unit on a side...
-- maps texture coordinates UV to cover a portion of a side;
-- thus multiple copies of *.png file may be visible


type rectangle is tagged private;

--for rectangular objects with faces aligned with the room:
procedure setrect( rect: in out rectangle;
xc,yc,zc, xr,yr,zr : float;
xm,xp, ym,yp, zm,zp : out float --helps to define KeepOut regions
);

--this version allows specification of 4 corners
--which allows oddly tilted surfaces...
--the outward normal is defined by the CCW ordering
--of p1, p2, p3, p4:
procedure setrect2( rect: in out rectangle;
x1,y1,z1, x2,y2,z2, x3,y3,z3, x4,y4,z4 : float;
su, sv: float := 1.0
);



procedure draw( rect: rectangle; vertbuff, uvbuff, elembuff : gluint );
procedure ldraw( rect: rectangle; vertbuff, uvbuff, normbuff, elembuff : gluint );

private

nvert : constant integer := 72;
nuv : constant integer := 48;
nelm : constant integer := 36;

type varray is array(1..nvert) of float;
type vap is access varray;

type tarray is array(1..nuv) of float;
type tap is access tarray;

type earray is array(1..nelm) of glushort;
type eap is access earray;

type rectangle is new ada.finalization.controlled with record
	vert, norm : vap;
	txuv : tap;
	elem : eap;
	nv,nt,ne: integer; --actuals (1/6 size for single surface)
end record;


procedure initialize(rect: in out rectangle);
procedure finalize(rect: in out rectangle);

end rectobj;

