
--
-- Copyright (C) 2024  <fastrgv@gmail.com>
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You may read the full text of the GNU General Public License
-- at <http://www.gnu.org/licenses/>.
--

with gl;  use gl;
with ada.finalization;
with unchecked_deallocation;



generic
	npts: integer; --# radial subdivisions, # angular subdivisions

package splashobj is 
-- for outward water splashes using radially-sliding textures


type splash is tagged private;

procedure setrect( rect: splash );

procedure update( 
	rect: in out splash; 
	radvel, etsec: float );


procedure draw( rect: splash; vertbuff, uvbuff, elembuff : gluint );

private

nvert : constant integer :=  8*npts*4; 
nuv   : constant integer :=  8*npts*4; 
nelm  : constant integer :=  6*npts*4; 

type varray is array(1..nvert) of float;
type vap is access varray;

type tarray is array(1..nuv) of float;
type tap is access tarray;

type earray is array(1..nelm) of glushort;
type eap is access earray;

type splash is new ada.finalization.controlled with record
	vert : vap;
	txuv : tap;
	elem : eap;
end record;


procedure initialize(rect: in out splash);
procedure finalize(rect: in out splash);

end splashobj;

